<?php
/**
 * HARC/Util/CsvParser.php
 *
 */

/**
 */
class HARC_Util_CsvParser
{
  protected $_field;
  protected $_lineNumber = 0;
  protected $_delimiter = ',';
  protected $_quote = '"';
  protected $_encoding = 'sjis-win, eucjp-win';
  protected $_maxLineLength = 65536;
  protected $_maxRecordLength;

  public function __construct(array $param = array()) {
    if(!empty($param['delimiter'])) {
      $this->_delimiter = $param['delimiter'];
    }
    if(!empty($param['quote'])) {
      $this->_quote = $param['quote'];
    }
    if(!empty($param['encoding'])) {
      $this->_encoding = $param['encoding'];
    }
    if(!empty($param['maxLineLength'])) {
      $this->_maxLineLength = $this->parseMemoryExpr($param['maxLineLength']);
    }
    if(empty($param['maxRecordLength'])) {
      $this->_maxRecordLength = $this->getIniMemoryLimit() / 4;
    } else {
      $this->_maxRecordLength = $this->parseMemoryExpr($param['maxRecordLimit']);
    }
  }

  public function parserHeader($fh) {
  }

  public function parseRecords($fh, $schema, $closure) {
  }

  public function parseRecord($fh) {
    $enc = $this->_encoding;
    $quote = $this->_quote;

    $start = $this->_lineNumber + 1;
    $buf = '';
    $len = $this->_maxLineLength;
    while($line = fgets($fh, $len)) {
      $this->_lineNumber += 1;

      if(strlen($line) >= $len - 1) {
        $lc = $this->_lineNumber;
        throw Exception("line buffer exhausted (line=$lc)");
      }
      $buf .= $line;
      if(strlen($buf) >= $this->_maxRecordLength) {
        $lc = $this->_lineNumber;
        throw Exception("record buffer exhausted (line=$start..$lc)");
      }

      $expr = mb_convert_encoding($buf, 'UTF-8', $enc);
      $c = substr_count($expr, $quote);
      if($c % 2 == 0) {
        return $this->splitRecord($expr);
      }
    }

    if(!empty($buf)) {
      throw Exception("unexpected EOF");
    }
    return false;
  }

  protected function splitRecord($expr) {
    $d = preg_quote($this->_delimiter);
    $q = preg_quote($this->_quote);

    $buf = preg_replace('/(?:\\r\\n|[\\r\\n])?$/', $d, trim($expr));
    $pattern = '/('.$q.'[^'.$q.']*(?:'.$q.$q.'[^'.$q.']*)*'.$q.'|[^'.$d.']*)'.$d.'/';
    preg_match_all($pattern, $buf, $matches);
    $fields = $matches[1];
    foreach ($fields as &$item) {
      $item = preg_replace('/^'.$q.'(.*)'.$q.'$/s', '$1', $item);
      $item = str_replace($q.$q, $q, $item);
    }
    return $fields;
  }

  private function getIniMemoryLimit() {
    $expr = ini_get('memory_limit');
    $mem = $this->parseMemoryExpr($expr);
    if($mem > 0) {
      return $mem;
    }
    return 32 * 1024 * 1024;
  }

  private function parseMemoryExpr($expr) {
    if(preg_match('/(\d+)([KkMmGg])?/', $expr, $matches)) {
      $base = intval($matches[1]);
      $radix = $matches[2];
      if($radix == 'G' || $radix == 'g') {
        return $base * 1024 * 1024 * 1024;
      }
      if($radix == 'M' || $radix == 'm') {
        return $base * 1024 * 1024;
      }
      if($radix == 'K' || $radix == 'k') {
        return $base * 1024;
      }
      return $base;
    }
    return false;
  }
}
