<?php

/**
 * 日付作成ユーティリティ
 * @author hunglead
 */
class DateUtil {

	/**
	 * 日付オブジェクトを作成
	 * @param string $strDate 日付文字列
	 */
	public static function createDateTime($strDate = null) {
		if (class_exists("DateTime")) {
			return new DateTime($strDate, new DateTimeZone("Asia/Tokyo"));
		} else {
			return new SimpleDateTime($strDate);
		}
	}
}

/**
 * DateTime簡易版（PHP5.1対応用）
 * @author hunglead
 */
class SimpleDateTime {


	private $time;

	/**
	 * コンストラクタ
	 * @param string $strDate 日付文字列
	 */
	public function __construct($strDate) {

		date_default_timezone_set("Asia/Tokyo");

		if ($strDate) {
			$this->time = strtotime($strDate);
		} else {
			$this->time = time();
		}
	}

	/**
	 * フォーマット
	 * @param string $format フォーマット形式
	 */
	public function format($format) {
		return date($format, $this->time);
	}

	/**
	 * 日付を変更
	 * @param string $modify 変更文字列
	 */
	public function modify($modify) {
		$this->time = strtotime($modify, $this->time);
		return $this;
	}
}